import uuid from 'uuid/v4';
import _ from 'lodash';
import { datasourceApi } from '../../../services/datasource';

const prefix = uuid();

const getMeta = (property, name, parsed = false, defaultValue) => {
  if (!property) {
    return defaultValue;
  }
  const names = _.castArray(name);
  const result = [];
  _.some(names, (v) => {
    const ret = property[v];
    if (!_.isUndefined(ret)) {
      result.push(parsed ? JSON.parse(ret) : ret);
      return true;
    }
    return false;
  });
  return result.length > 0 ? result[0] : defaultValue;
};

const getRowIndex = (meta) => {
  return _(meta.properties || [])
    .filter(property => !property.skip)
    .findIndex(property => property.key);
};

const makeColumns = (meta) => {
  return (meta.properties || [])
    .filter(property => !property.skip)
    .map((property, idx) => ({
      title: getMeta(property, ['label', 'title'], false, property.name),
      dataIndex: idx,
      key: property.name,
      filterType: getMeta(property, 'filterType'),
      filterEnums: getMeta(property, 'filterEnums', true),
      order: _.toNumber(getMeta(property, 'order', false, 0)),
      group: getMeta(property, 'group'),
    }));
};

const getArrayData = ({ dataType, arrayData, singularData }, meta) => {
  if (dataType === 'TABLE') {
    return (arrayData && arrayData.map(v => _.toPlainObject(v))) || [];
  } else if (dataType === 'PROPERTIES') {
    const data = [];
    (meta.properties || [])
      .filter(property => !property.skip)
      .forEach((property) => {
        data.push((singularData || {})[property.name]);
      });
    return [_.toPlainObject(data)];
  } else {
    throw new Error(`Unsupported data type: ${dataType}`);
  }
};

const modelCreator = () => {
  const name = 'model';
  const namespace = _.uniqueId(prefix);
  return {
    name,
    model: {
      namespace,
      state: {
        columns: [],
        rowKey: '',
        num: 0,
        list: [],
      },
      reducers: {
        queryMetaSuccess(state, { payload: { rowKey, columns } }) {
          return {
            ...state,
            rowKey,
            columns,
          }
        },
        queryCountSuccess(state, { payload: num }) {
          return {
            ...state,
            num,
          };
        },
        queryTasksSuccess(state, { payload: list }) {
          return {
            ...state,
            list,
          };
        },
      },
      effects: {
        *fetchData({ payload: { coordinate, pst, psz, params, filters} }, { put, call }) {
          const options = { pst, psz, params, filters };
          const api = datasourceApi(coordinate);
          const meta = yield call(api.meta);
          const rowKey = `${getRowIndex(meta)}`;
          yield put({ type: 'queryMetaSuccess', payload: { rowKey, columns: makeColumns(meta) } });
          const num = yield call(api.count, options);
          yield put({ type: 'queryCountSuccess', payload: num });
          const dsb = yield call(api.query, options);
          yield put({ type: 'queryTasksSuccess', payload: getArrayData(dsb, meta) });
        },
      },
      subscriptions: {},
    },
  };
};

export default modelCreator;
